'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const axios = require('axios');
const freeBrandsSvgIcons = require('@fortawesome/free-brands-svg-icons');
const freeSolidSvgIcons = require('@fortawesome/free-solid-svg-icons');
const reactFontawesome = require('@fortawesome/react-fontawesome');
const typeAnalyzer = require('@previewjs/type-analyzer');
const windowSize = require('@react-hook/window-size');
const clsx = require('clsx');
const mobxReactLite = require('mobx-react-lite');
const React$1 = require('react');
const Editor = require('@monaco-editor/react');
const typescript = require('typescript');
const framerMotion = require('framer-motion');
const mobx = require('mobx');
const api = require('@previewjs/api');
const iframe = require('@previewjs/iframe');
const assertNever = require('assert-never');
const serializableValues = require('@previewjs/serializable-values');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

const axios__default = /*#__PURE__*/_interopDefaultLegacy(axios);
const clsx__default = /*#__PURE__*/_interopDefaultLegacy(clsx);
const React__default = /*#__PURE__*/_interopDefaultLegacy(React$1);
const Editor__default = /*#__PURE__*/_interopDefaultLegacy(Editor);
const assertNever__default = /*#__PURE__*/_interopDefaultLegacy(assertNever);

function createLocalApi(url) {
  return new LocalApi(url);
}
class LocalApi {
  constructor(url) {
    if (!url.endsWith("/")) {
      url += "/";
    }
    this.url = url;
  }
  async request(...[endpoint, request]) {
    const { data } = await axios__default.post(`${this.url}${endpoint.path}`, request);
    if (data.kind === "error") {
      throw new Error(data.message);
    }
    return data.response;
  }
}

function createWebApi(url) {
  return new WebApi(url);
}
class WebApi {
  constructor(url) {
    if (!url.endsWith("/")) {
      url += "/";
    }
    this.url = url;
  }
  async request(...[endpoint, request]) {
    const { data } = await axios__default.post(`${this.url}${endpoint.path}`, request);
    return data;
  }
}

const FilePath = ({ filePath }) => {
  return /* @__PURE__ */ React__default.createElement("div", {
    className: "truncate flex flex-row items-center px-2 py-1 bg-gray-100 text-gray-800 rounded-md"
  }, /* @__PURE__ */ React__default.createElement("div", {
    className: "mr-2"
  }, /* @__PURE__ */ React__default.createElement(reactFontawesome.FontAwesomeIcon, {
    icon: freeSolidSvgIcons.faFileCode
  })), /* @__PURE__ */ React__default.createElement("div", {
    className: "text-left truncate",
    style: { direction: "rtl" }
  }, filePath));
};

const Container$2 = ({ children }) => /* @__PURE__ */ React__default.createElement("div", {
  className: "text-sm bg-gray-900 filter drop-shadow"
}, children);
const Row = ({
  children,
  className
}) => /* @__PURE__ */ React__default.createElement("div", {
  className: clsx__default(["flex flex-row items-center p-1.5", className])
}, children);
const Header = Container$2;
Header.Row = Row;

const Link = (props) => {
  const onClick = React$1.useCallback((event) => {
    if (props.href) {
      openUrl(props.href);
      event.preventDefault();
    }
    return props.onClick && props.onClick(event);
  }, [props]);
  return /* @__PURE__ */ React__default.createElement("a", {
    ...props,
    onClick
  });
};
function openUrl(url) {
  if (window.openInExternalBrowser) {
    window.openInExternalBrowser(url);
    return;
  }
  if (navigator.userAgent.includes(" Code/")) {
    window.parent.postMessage({
      command: "open-browser",
      url
    }, "*");
    return;
  }
  window.open(url, "_blank");
}

async function revalidateMonacoEditor(monaco, editor) {
  const model = editor.getModel();
  if (!model) {
    return;
  }
  const getWorker = await monaco.languages.typescript.getTypeScriptWorker();
  const worker = await getWorker(model.uri);
  const diagnostics = (await Promise.all([
    worker.getSyntacticDiagnostics(model.uri.toString()),
    worker.getSemanticDiagnostics(model.uri.toString())
  ])).reduce((a, it) => a.concat(it));
  const markers = diagnostics.map((d) => {
    const startPos = d.start || 0;
    const length = d.length || 0;
    const start = model.getPositionAt(startPos);
    const end = model.getPositionAt(startPos + length);
    return {
      severity: monaco.MarkerSeverity.Error,
      startLineNumber: start.lineNumber,
      startColumn: start.column,
      endLineNumber: end.lineNumber,
      endColumn: end.column,
      message: typescript.flattenDiagnosticMessageText(d.messageText, "\n")
    };
  });
  const owner = model.getLanguageId();
  monaco.editor.setModelMarkers(model, owner, markers);
}

Editor.loader.config({
  paths: {
    vs: "/monaco-editor/min/vs"
  }
});
const PropsEditor = ({
  documentId,
  source,
  typeDeclarationsSource,
  onUpdate
}) => {
  const editorRef = React$1.useRef();
  const modelRef = React$1.useRef();
  const [mounted, setMounted] = React$1.useState(false);
  const onChange = React$1.useCallback((value) => {
    if (value !== void 0 && value !== source) {
      onUpdate(value);
    }
  }, [onUpdate, source]);
  React$1.useEffect(() => {
    const editor = editorRef.current;
    if (!editor) {
      return;
    }
    editor.setPosition({
      lineNumber: 1,
      column: Number.MAX_SAFE_INTEGER
    });
    editor.setScrollTop(0);
  }, [documentId]);
  const monaco = Editor.useMonaco();
  React$1.useEffect(() => {
    if (monaco && typeDeclarationsSource) {
      const libUri = monaco.Uri.parse("ts:filename/component.d.tsx");
      const existingModel = monaco.editor.getModel(libUri);
      if (existingModel) {
        existingModel.setValue(typeDeclarationsSource);
      } else {
        monaco.editor.createModel(typeDeclarationsSource, "typescript", libUri);
      }
      if (editorRef.current) {
        revalidateMonacoEditor(monaco, editorRef.current).catch(console.warn);
      }
    }
  }, [monaco, typeDeclarationsSource]);
  React$1.useEffect(() => {
    const model = modelRef.current;
    if (!model || !source || model.getValue() === source) {
      return;
    }
    model.setValue(source);
  }, [source, mounted]);
  const beforeMount = React$1.useCallback((monaco2) => {
    monaco2.languages.typescript.typescriptDefaults.setCompilerOptions({
      target: monaco2.languages.typescript.ScriptTarget.ESNext,
      noEmit: true,
      jsx: monaco2.languages.typescript.JsxEmit.Preserve
    });
  }, []);
  const onMount = React$1.useCallback((editor, monaco2) => {
    const usageUri = monaco2.Uri.parse("file:///usage.d.tsx");
    const existingModel = monaco2.editor.getModel(usageUri);
    if (existingModel) {
      modelRef.current = existingModel;
      existingModel.setValue(source || "");
    } else {
      modelRef.current = monaco2.editor.createModel(source || "", "typescript", monaco2.Uri.parse("file:///usage.d.tsx"));
    }
    editorRef.current = editor;
    editor.setModel(modelRef.current);
    for (const model of monaco2.editor.getModels()) {
      if (model.uri.toString().startsWith("inmemory://model/")) {
        model.dispose();
      }
    }
    setMounted(true);
  }, [source]);
  return /* @__PURE__ */ React__default.createElement(React__default.Fragment, null, /* @__PURE__ */ React__default.createElement(Editor__default, {
    theme: "vs-dark",
    defaultLanguage: "typescript",
    onChange,
    options: {
      minimap: {
        enabled: false
      },
      overviewRulerLanes: 0,
      lineNumbers: "off",
      folding: false,
      renderLineHighlight: "none",
      lineDecorationsWidth: 8,
      lineNumbersMinChars: 0,
      scrollbar: {
        vertical: "hidden"
      },
      scrollBeyondLastLine: false,
      padding: {
        top: 8,
        bottom: 8
      },
      fontSize: 12
    },
    beforeMount,
    onMount
  }), /* @__PURE__ */ React__default.createElement("div", {
    id: "editor-content",
    "data-value": source,
    style: { display: "none" }
  }), mounted && /* @__PURE__ */ React__default.createElement("div", {
    id: "editor-ready"
  }));
};

const SmallLogo = (props) => {
  return /* @__PURE__ */ React.createElement(Link, {
    className: "inline-flex items-center text-gray-50 ml-2 pl-0.5 pr-1.5 py-0.5 text-sm font-semibold rounded whitespace-nowrap",
    target: "_blank",
    href: props.href,
    title: props.title
  }, /* @__PURE__ */ React.createElement("svg", {
    width: "512px",
    height: "512px",
    viewBox: "0 0 512 512",
    xmlns: "http://www.w3.org/2000/svg",
    xmlnsXlink: "http://www.w3.org/1999/xlink",
    className: "h-6 w-6 mr-1"
  }, /* @__PURE__ */ React.createElement("defs", null, /* @__PURE__ */ React.createElement("path", {
    d: "M178.998,315 L179,439 L105,439 L105,46 L240,46 L240,46.004 L241.724203,46.0180192 C314.980631,47.2057605 374,106.960524 374,180.5 C374,254.281294 314.591849,314.187071 241.001081,314.991796 L241,315 L178.998,315 Z M240.5,109 C200.459356,109 168,141.459356 168,181.5 C168,221.540644 200.459356,254 240.5,254 C280.540644,254 313,221.540644 313,181.5 C313,141.459356 280.540644,109 240.5,109 Z",
    id: "path-1"
  }), /* @__PURE__ */ React.createElement("filter", {
    x: "-9.3%",
    y: "-10.2%",
    width: "137.2%",
    height: "122.9%",
    filterUnits: "objectBoundingBox",
    id: "filter-2"
  }, /* @__PURE__ */ React.createElement("feOffset", {
    dx: "30",
    dy: "5",
    in: "SourceAlpha",
    result: "shadowOffsetOuter1"
  }), /* @__PURE__ */ React.createElement("feGaussianBlur", {
    stdDeviation: "10",
    in: "shadowOffsetOuter1",
    result: "shadowBlurOuter1"
  }), /* @__PURE__ */ React.createElement("feColorMatrix", {
    values: "0 0 0 0 0   0 0 0 0 0   0 0 0 0 0  0 0 0 0.253425045 0",
    type: "matrix",
    in: "shadowBlurOuter1"
  }))), /* @__PURE__ */ React.createElement("g", {
    id: "logo",
    stroke: "none",
    strokeWidth: "1",
    fill: "none",
    fillRule: "evenodd"
  }, /* @__PURE__ */ React.createElement("g", {
    id: "Group",
    transform: "translate(17.000000, 17.000000)"
  }, /* @__PURE__ */ React.createElement("rect", {
    id: "Rectangle",
    fill: "#3B83F6",
    x: "4",
    y: "4",
    width: "470",
    height: "470",
    rx: "69"
  }), /* @__PURE__ */ React.createElement("g", {
    id: "Oval"
  }, /* @__PURE__ */ React.createElement("use", {
    fill: "black",
    fillOpacity: "1",
    filter: "url(#filter-2)",
    xlinkHref: "#path-1"
  }), /* @__PURE__ */ React.createElement("use", {
    fill: "#FFFFFF",
    fillRule: "evenodd",
    xlinkHref: "#path-1"
  }))))), props.label);
};

const TabbedPanel = (props) => {
  const [currentTabKey, setCurrentTabKey] = React$1.useState(props.defaultTabKey);
  const currentTab = props.tabs.find((tab) => tab.key === currentTabKey);
  return /* @__PURE__ */ React__default.createElement("div", null, /* @__PURE__ */ React__default.createElement("div", {
    className: "flex flex-row flex-wrap items-center bg-white shadow-inner"
  }, props.tabs.map((tab) => /* @__PURE__ */ React__default.createElement("button", {
    key: tab.key,
    title: tab.label,
    onClick: () => {
      setCurrentTabKey(tab.key === currentTabKey ? null : tab.key);
    },
    className: clsx__default([
      "panel-tab",
      "px-3 py-2 self-stretch text-sm font-medium flex flex-row items-center",
      tab.key === currentTabKey ? "panel-tab-selected bg-gray-200 text-gray-900" : "text-gray-400"
    ])
  }, /* @__PURE__ */ React__default.createElement(reactFontawesome.FontAwesomeIcon, {
    icon: tab.icon
  }), /* @__PURE__ */ React__default.createElement("div", {
    className: clsx__default(["ml-2"])
  }, tab.label), tab.notificationCount > 0 && /* @__PURE__ */ React__default.createElement("span", {
    className: "notification-count bg-red-600 text-white p-2 rounded-full inline-flex items-center justify-center h-5 min-w-5 ml-1 text-xs"
  }, tab.notificationCount)))), currentTab && /* @__PURE__ */ React__default.createElement("div", {
    className: "flex flex-col bg-gray-200",
    style: { height: props.height }
  }, currentTab.panel));
};

function decodeComponentId(componentId) {
  const colonPosition = componentId.lastIndexOf(":");
  if (colonPosition === -1) {
    return {
      currentFilePath: componentId,
      component: null
    };
  }
  const currentFilePath = componentId.slice(0, colonPosition);
  const componentKey = componentId.slice(colonPosition + 1);
  const slashPosition = componentKey.lastIndexOf("/");
  if (slashPosition === -1) {
    return {
      currentFilePath,
      component: {
        filePath: currentFilePath,
        name: componentKey
      }
    };
  }
  const dirPath = currentFilePath.slice(0, currentFilePath.lastIndexOf("/"));
  const siblingFileName = componentKey.slice(0, slashPosition);
  const componentName = componentKey.slice(slashPosition + 1);
  return {
    currentFilePath,
    component: {
      filePath: `${dirPath}/${siblingFileName}`,
      name: componentName
    }
  };
}

function generatePropsTypeDeclarations(typeName, types) {
  return `declare let properties: ${typeName.split(":")[1]};

${typeAnalyzer.generateTypeDeclarations([typeName], types)}
`;
}

const Container$1 = ({
  children
}) => {
  return /* @__PURE__ */ React__default.createElement("div", {
    className: "absolute overflow-hidden bottom-0 right-0 z-50"
  }, children);
};

const Item$1 = (props) => {
  return /* @__PURE__ */ React__default.createElement(framerMotion.motion.div, {
    animate: { x: "100%" },
    transition: { delay: 3 },
    onAnimationComplete: props.onAnimationComplete
  }, /* @__PURE__ */ React__default.createElement(framerMotion.motion.div, {
    animate: { x: 0, marginBottom: 0 },
    initial: { x: "100%", marginBottom: -100 },
    transition: { type: "tween" }
  }, /* @__PURE__ */ React__default.createElement("div", {
    className: "action-log bg-blue-700 text-blue-100 bg-opacity-80 p-2 m-2 border-2 border-blue-300"
  }, props.children)));
};

const PopOver = {
  Container: Container$1,
  Item: Item$1
};

const ActionLogs = mobxReactLite.observer(({ state }) => {
  return /* @__PURE__ */ React__default.createElement(PopOver.Container, null, state.logs.map((item) => /* @__PURE__ */ React__default.createElement(PopOver.Item, {
    key: item.key,
    onAnimationComplete: item.remove
  }, item.action.type === "fn" ? /* @__PURE__ */ React__default.createElement(React__default.Fragment, null, "Function prop invoked: ", /* @__PURE__ */ React__default.createElement("b", null, item.action.path)) : /* @__PURE__ */ React__default.createElement(React__default.Fragment, null, "Redirect prevented:", " ", /* @__PURE__ */ React__default.createElement(Link, {
    href: item.action.path
  }, item.action.path)), item.count > 1 && ` (x${item.count})`)));
});

class ActionLogsState {
  constructor() {
    this.logs = [];
    mobx.makeAutoObservable(this);
  }
  onAction(action) {
    const key = `${Date.now()}-${action.type}-${action.path}`;
    const existingActionLog = this.logs.find((a) => a.action.path === action.path);
    if (existingActionLog) {
      existingActionLog.count += 1;
    } else {
      const item = {
        key,
        action,
        count: 1,
        remove: () => this.removeItem(item)
      };
      this.logs.push(item);
    }
  }
  removeItem(item) {
    this.logs = this.logs.filter((a) => a.key !== item.key);
  }
}

const Container = (props) => {
  const scrollableContainerRef = React$1.useRef(null);
  React$1.useEffect(() => {
    if (!scrollableContainerRef.current) {
      return;
    }
    scrollableContainerRef.current.scroll({
      behavior: "smooth",
      top: scrollableContainerRef.current.scrollHeight
    });
  });
  return /* @__PURE__ */ React__default.createElement("div", {
    id: "console-container",
    className: "flex-grow flex flex-col min-h-0"
  }, /* @__PURE__ */ React__default.createElement("div", {
    ref: scrollableContainerRef,
    className: "flex flex-col overflow-y-auto"
  }, props.children), props.children.length > 0 && /* @__PURE__ */ React__default.createElement("button", {
    id: "clear-console-button",
    className: "p-2 text-gray-400 border-t-2 border-gray-100 hover:text-gray-600",
    onClick: props.onClear
  }, /* @__PURE__ */ React__default.createElement(reactFontawesome.FontAwesomeIcon, {
    icon: freeSolidSvgIcons.faTimesCircle
  }), " Clear all"));
};

const dateFormat = new Intl.DateTimeFormat("default", {
  hour: "numeric",
  minute: "numeric",
  second: "numeric"
});
const Item = (props) => {
  return /* @__PURE__ */ React__default.createElement("code", {
    className: clsx__default([
      "console-item",
      `console-item-${props.level}`,
      "block p-2 text-xs whitespace-pre-wrap",
      (props.level === "log" || props.level === "info") && "bg-blue-50 odd:bg-blue-100 text-blue-400",
      props.level === "warn" && "bg-orange-50 odd:bg-orange-100 text-orange-400",
      props.level === "error" && "bg-red-50 odd:bg-red-100 text-red-600"
    ])
  }, "[", dateFormat.format(props.timestamp), "] ", props.message, props.suggestion && /* @__PURE__ */ React__default.createElement(Link, {
    className: "block mt-2 p-2 bg-white shadow text-blue-800 font-sans",
    href: props.suggestion.url
  }, /* @__PURE__ */ React__default.createElement(reactFontawesome.FontAwesomeIcon, {
    icon: freeSolidSvgIcons.faCircleQuestion,
    className: "mr-2"
  }), props.suggestion.message));
};

const ConsoleLogs = {
  Container,
  Item
};

const ConsolePanel = mobxReactLite.observer(({ state }) => {
  return /* @__PURE__ */ React__default.createElement(ConsoleLogs.Container, {
    onClear: state.onClear.bind(state)
  }, state.logs.map((log) => /* @__PURE__ */ React__default.createElement(ConsoleLogs.Item, {
    key: `${log.timestamp}-${log.message}`,
    ...log
  })));
});

class ConsolePanelState {
  constructor() {
    this.logs = [];
    mobx.makeAutoObservable(this);
  }
  onConsoleMessage(event) {
    if (this.logs.find((log) => event.level === log.level && event.message === log.message && Math.abs(event.timestamp - log.timestamp) < 1e3)) {
      return;
    }
    this.logs.push({
      ...event,
      suggestion: generateSuggestion(event.message)
    });
  }
  onClear() {
    this.logs = [];
  }
  get unreadCount() {
    return this.logs.filter((message) => isNotifiable(message.level)).length;
  }
}
function isNotifiable(level) {
  switch (level) {
    case "warn":
    case "error":
      return true;
    default:
      return false;
  }
}
function generateSuggestion(message) {
  if (message.includes(`Failed to resolve import `)) {
    const match = message.match(/Failed to resolve import "((@[a-zA-Z0-9\\-]+\/[a-zA-Z0-9\\-]+)|[a-zA-Z0-9\\-]+)"/);
    const url = "https://previewjs.com/docs/config/aliases";
    return {
      message: match ? `Perhaps you need to install "${match[1]}" or configure aliases? See ${url}` : `Perhaps you need to install a peer dependency or configure aliases? See ${url}`,
      url
    };
  }
  if (message.includes(".svg?import")) {
    const url = "https://previewjs.com/docs/config/svgr";
    return {
      message: `Read ${url} to configure SVGR for your project.`,
      url
    };
  }
  return void 0;
}

const LinkBanner = (props) => {
  const { onDismiss } = props;
  const dismiss = React$1.useCallback((e) => {
    e.stopPropagation();
    e.preventDefault();
    if (onDismiss) {
      onDismiss();
    }
  }, [onDismiss]);
  return /* @__PURE__ */ React__default.createElement(Link, {
    className: clsx__default([
      "flex flex-row items-center px-2 py-1",
      props.type === "warn" && "bg-red-300",
      props.type === "info" && "bg-blue-300"
    ]),
    href: props.href
  }, /* @__PURE__ */ React__default.createElement("div", {
    className: "m-2"
  }, props.message), /* @__PURE__ */ React__default.createElement("div", {
    className: "flex-grow"
  }), /* @__PURE__ */ React__default.createElement("button", {
    className: clsx__default([
      "px-2 py-1 rounded-md font-bold",
      props.type === "warn" && "bg-red-50",
      props.type === "info" && "bg-red-50"
    ])
  }, props.buttonLabel), onDismiss && /* @__PURE__ */ React__default.createElement("button", {
    className: "ml-1 px-2 py-1",
    onClick: dismiss
  }, "Dismiss"));
};

const UpdateBanner = mobxReactLite.observer(({ state: { update, onUpdateDismissed } }) => {
  if (!update) {
    return null;
  }
  return /* @__PURE__ */ React__default.createElement(LinkBanner, {
    type: update.required ? "warn" : "info",
    href: update.url,
    message: update.bannerMessage,
    buttonLabel: "Update now",
    onDismiss: update.required ? void 0 : onUpdateDismissed
  });
});

class UpdateBannerState {
  constructor(webApi, persistedStateController) {
    this.webApi = webApi;
    this.persistedStateController = persistedStateController;
    this.checkVersionResponse = null;
    this.onUpdateDismissed = async () => {
      await this.persistedStateController.update({
        updateDismissed: {
          timestamp: Date.now()
        }
      });
    };
    mobx.makeAutoObservable(this);
  }
  async start(appInfo) {
    try {
      const checkVersionResponse = await this.webApi.request(api.webEndpoints.CheckVersion, {
        appInfo
      });
      mobx.runInAction(() => {
        this.checkVersionResponse = checkVersionResponse;
      });
    } catch (e) {
      console.warn(e);
    }
  }
  get update() {
    const update = this.checkVersionResponse?.update;
    const persistedState = this.persistedStateController.state;
    if (!update?.available || !persistedState) {
      return null;
    }
    const dismissedAt = persistedState.updateDismissed?.timestamp;
    if (!update.required && dismissedAt && Date.now() < dismissedAt + 24 * 60 * 60 * 1e3) {
      return null;
    }
    return update;
  }
}

const Preview = mobxReactLite.observer(({
  state,
  appLabel,
  headerAddon,
  subheader,
  footer,
  panelTabs,
  PropsPanel = DefaultPropsPanel,
  viewport
}) => {
  const panelHeight = windowSize.useWindowHeight() * 0.3;
  if (!state.reachable) {
    return /* @__PURE__ */ React__default.createElement("div", {
      className: "h-screen bg-gray-700 text-gray-100 p-2 grid place-items-center text-lg"
    }, /* @__PURE__ */ React__default.createElement("div", {
      id: "app-error",
      className: "p-4 rounded-lg text-red-800 bg-red-50 filter drop-shadow-lg"
    }, "Server disconnected. Is Preview.js still running?"));
  }
  const tabs = [];
  if (state.component) {
    if ((state.component.variantKey === null || state.component.variantKey === "custom") && state.component.details) {
      const { props } = state.component.details;
      const { propsTypeName, types } = props.propsType;
      const propsType = types[propsTypeName]?.type || typeAnalyzer.UNKNOWN_TYPE;
      const isEmptyProps = propsType.kind === "object" && Object.entries(propsType.fields).length === 0;
      if (!isEmptyProps && !props.isStory) {
        tabs.push({
          label: "Properties",
          key: "props",
          icon: freeSolidSvgIcons.faCode,
          notificationCount: 0,
          panel: /* @__PURE__ */ React__default.createElement(PropsPanel, {
            componentName: state.component.name,
            propsType: typeAnalyzer.namedType(propsTypeName),
            types,
            source: props.invocationSource,
            onChange: state.updateProps.bind(state),
            onReset: props.isDefaultInvocationSource ? void 0 : state.resetProps.bind(state),
            codeEditor: /* @__PURE__ */ React__default.createElement(PropsEditor, {
              documentId: state.component.componentId,
              onUpdate: state.updateProps.bind(state),
              source: props.invocationSource,
              typeDeclarationsSource: generatePropsTypeDeclarations(propsTypeName, types)
            })
          })
        });
      }
    }
    tabs.push({
      label: "Console",
      key: "console",
      icon: freeSolidSvgIcons.faTerminal,
      notificationCount: state.consoleLogs.unreadCount,
      panel: /* @__PURE__ */ React__default.createElement(ConsolePanel, {
        state: state.consoleLogs
      })
    });
  }
  if (panelTabs) {
    tabs.push(...panelTabs);
  }
  return /* @__PURE__ */ React__default.createElement("div", {
    className: "flex flex-col h-screen overflow-hidden bg-white"
  }, /* @__PURE__ */ React__default.createElement(ActionLogs, {
    state: state.actionLogs
  }), /* @__PURE__ */ React__default.createElement(Header, null, /* @__PURE__ */ React__default.createElement(Header.Row, null, headerAddon, state.component?.componentId && /* @__PURE__ */ React__default.createElement(React__default.Fragment, null, /* @__PURE__ */ React__default.createElement(FilePath, {
    key: "file",
    filePath: decodeComponentId(state.component.componentId).currentFilePath
  }), /* @__PURE__ */ React__default.createElement(Link, {
    href: document.location.href,
    target: "_blank",
    title: "Open in new tab",
    className: "text-gray-500 hover:text-gray-200 ml-2 text-lg"
  }, /* @__PURE__ */ React__default.createElement(reactFontawesome.FontAwesomeIcon, {
    icon: freeSolidSvgIcons.faExternalLink,
    fixedWidth: true
  }))), /* @__PURE__ */ React__default.createElement("div", {
    className: "flex-grow"
  }), /* @__PURE__ */ React__default.createElement(SmallLogo, {
    href: "https://previewjs.com/docs",
    label: appLabel,
    title: state.appInfo?.version && `v${state.appInfo.version}`
  }), /* @__PURE__ */ React__default.createElement(Link, {
    className: "ml-2 text-xl text-[#1DA1F2] hover:text-white",
    href: "https://twitter.com/previewjs",
    target: "_blank",
    title: "Follow Preview.js on Twitter"
  }, /* @__PURE__ */ React__default.createElement(reactFontawesome.FontAwesomeIcon, {
    icon: freeBrandsSvgIcons.faTwitter,
    fixedWidth: true
  })), /* @__PURE__ */ React__default.createElement(Link, {
    className: "ml-2 text-xl bg-[#333] text-white rounded-md hover:bg-white hover:text-[#333]",
    href: "https://github.com/fwouts/previewjs",
    target: "_blank",
    title: "Star Preview.js on GitHub"
  }, /* @__PURE__ */ React__default.createElement(reactFontawesome.FontAwesomeIcon, {
    icon: freeBrandsSvgIcons.faGithub,
    fixedWidth: true
  }))), subheader && /* @__PURE__ */ React__default.createElement(Header.Row, {
    className: "bg-gray-100"
  }, subheader)), /* @__PURE__ */ React__default.createElement(UpdateBanner, {
    state: state.updateBanner
  }), state.component ? viewport : /* @__PURE__ */ React__default.createElement("div", {
    id: "no-selection",
    className: "flex-grow bg-gray-700 text-gray-100 p-2 grid place-items-center text-lg"
  }, "Please select a component to preview."), /* @__PURE__ */ React__default.createElement(TabbedPanel, {
    defaultTabKey: "props",
    tabs,
    height: panelHeight
  }), footer);
});
const DefaultPropsPanel = ({
  codeEditor,
  onReset
}) => {
  return /* @__PURE__ */ React__default.createElement(React$1.Fragment, null, /* @__PURE__ */ React__default.createElement("button", {
    id: "editor-refresh-button",
    className: clsx__default([
      "absolute right-0 m-2 p-2 bg-gray-500 rounded-md z-50",
      onReset ? "bg-opacity-40 text-white cursor-pointer hover:bg-gray-400" : "bg-opacity-25 text-gray-500"
    ]),
    title: "Reset properties",
    disabled: !onReset,
    onClick: onReset
  }, /* @__PURE__ */ React__default.createElement(reactFontawesome.FontAwesomeIcon, {
    icon: freeSolidSvgIcons.faUndo
  })), codeEditor);
};

const Pill = React$1.forwardRef((props, ref) => {
  return /* @__PURE__ */ React__default.createElement("div", {
    ref,
    className: "flex flex-row shrink-0 items-stretch cursor-pointer rounded-full overflow-hidden bg-white border-2 border-gray-300 text-gray-800",
    onClick: props.onClick
  }, /* @__PURE__ */ React__default.createElement("span", {
    id: "component-label",
    className: "m-1 px-2"
  }, props.loading ? /* @__PURE__ */ React__default.createElement(reactFontawesome.FontAwesomeIcon, {
    className: "mr-2 animate-spin",
    icon: freeSolidSvgIcons.faSpinner,
    fixedWidth: true
  }) : /* @__PURE__ */ React__default.createElement(reactFontawesome.FontAwesomeIcon, {
    className: "mr-2",
    icon: props.icon,
    fixedWidth: true
  }), props.label), props.buttons?.map((v) => {
    const selected = props.selectedButtonKey === v.key;
    return /* @__PURE__ */ React__default.createElement("div", {
      key: v.key,
      className: clsx__default([
        "button",
        "flex flex-row items-center px-3 font-light",
        selected ? "text-gray-100 bg-gray-600" : "text-gray-800 hover:bg-gray-200"
      ]),
      id: selected ? "selected-button" : void 0,
      onClick: (e) => {
        if (props.onButtonClicked) {
          props.onButtonClicked(v.key);
        }
        e.stopPropagation();
      }
    }, v.label);
  }));
});

const Selection = mobxReactLite.observer(React$1.forwardRef(({
  state,
  icon = freeSolidSvgIcons.faCode
}, ref) => {
  if (!state.component) {
    return null;
  }
  return /* @__PURE__ */ React__default.createElement(Pill, {
    ref,
    key: "component",
    icon,
    label: state.component.name,
    loading: !state.component.details?.variants,
    buttons: state.component.details?.variants?.filter((v) => !v.isEditorDriven) || void 0,
    selectedButtonKey: state.component.variantKey,
    onClick: () => state.setVariant("custom"),
    onButtonClicked: state.setVariant.bind(state)
  });
}));

class PersistedStateController {
  constructor(localApi) {
    this.localApi = localApi;
    this.persistedState = null;
    mobx.makeAutoObservable(this);
  }
  async start() {
    const persistedState = await this.localApi.request(api.localEndpoints.GetState);
    mobx.runInAction(() => {
      this.persistedState = persistedState;
    });
  }
  get state() {
    return this.persistedState;
  }
  async update(stateUpdate) {
    mobx.runInAction(() => {
      this.persistedState = {
        ...this.persistedState,
        ...stateUpdate
      };
    });
    await this.localApi.request(api.localEndpoints.UpdateState, stateUpdate);
  }
}

function extractFunctionKeys(propsType, types) {
  propsType = resolveType(propsType);
  if (propsType.kind !== "object") {
    return [];
  }
  return Object.entries(propsType.fields).filter(([, propertyType]) => resolveType(propertyType).kind === "function").map(([propertyName]) => propertyName);
  function resolveType(type) {
    if (type.kind === "name") {
      const resolved = types[type.name];
      if (resolved) {
        return typeAnalyzer.evaluateType(resolved, []);
      }
    }
    return type;
  }
}

function generateDefaultProps(propsType, types) {
  propsType = resolveType(propsType);
  if (propsType.kind !== "object") {
    return "{}";
  }
  const propKeys = /* @__PURE__ */ new Set();
  let text = "";
  text += "{";
  for (const [propertyName, propertyType] of Object.entries(propsType.fields)) {
    const resolvedPropertyType = resolveType(propertyType);
    if (resolvedPropertyType.kind === "function") {
      propKeys.add(propertyName);
      text += `"${propertyName.replace(/"/g, '\\"')}": ${serializableValues.serializableValueToJavaScript(serializableValues.generateSerializableValue(resolvedPropertyType, types))},`;
    }
  }
  text += "}";
  return text;
  function resolveType(type) {
    if (type.kind === "name") {
      const resolved = types[type.name];
      if (resolved) {
        return typeAnalyzer.evaluateType(resolved, []);
      }
    }
    return type;
  }
}

function generateInvocation(propsType, providedKeys, collected) {
  const providedKeySet = new Set(providedKeys);
  [propsType] = typeAnalyzer.dereferenceType(propsType, collected, []);
  if (propsType.kind === "object") {
    propsType = typeAnalyzer.objectType(Object.fromEntries(Object.entries(propsType.fields).filter(([fieldName]) => !providedKeySet.has(fieldName))));
  }
  const value = serializableValues.generateSerializableValue(propsType, collected);
  let valueSource = serializableValues.serializableValueToJavaScript(value);
  if (valueSource === "undefined") {
    valueSource = "{}";
  }
  return `properties = ${valueSource};`;
}

function preparePropsType(name, type, argKeys, collected) {
  const args = new Set(argKeys);
  const types = { ...collected };
  let rootTypeName;
  if (type.kind === "name") {
    rootTypeName = type.name;
  } else {
    rootTypeName = `:${name}Props`;
    types[rootTypeName] = { type, parameters: {} };
  }
  let actualObjectTypeName = rootTypeName;
  const previousTypeNames = new Set(rootTypeName);
  while (true) {
    const type2 = types[actualObjectTypeName]?.type;
    if (type2?.kind === "name") {
      if (previousTypeNames.has(type2.name)) {
        break;
      }
      previousTypeNames.add(type2.name);
      actualObjectTypeName = type2.name;
    } else {
      break;
    }
  }
  const propsType = types[actualObjectTypeName];
  if (propsType?.type.kind === "object") {
    types[actualObjectTypeName] = {
      type: typeAnalyzer.objectType(Object.fromEntries(Object.entries(propsType.type.fields).map(([fieldName, fieldType]) => {
        if (fieldType.kind === "function" || args.has(fieldName)) {
          fieldType = typeAnalyzer.optionalType(fieldType);
        }
        return [fieldName, fieldType];
      }))),
      parameters: propsType.parameters
    };
  }
  return {
    propsTypeName: rootTypeName,
    types
  };
}

class ComponentProps {
  constructor(localApi, filePath, componentName, cachedInvocationSource) {
    this.localApi = localApi;
    this.filePath = filePath;
    this.componentName = componentName;
    this.computePropsResponse = {
      args: [],
      isStory: false,
      types: {
        props: typeAnalyzer.UNKNOWN_TYPE,
        all: {}
      }
    };
    this._invocationSource = cachedInvocationSource;
    mobx.makeAutoObservable(this);
  }
  async refresh() {
    const response = await this.localApi.request(api.localEndpoints.ComputeProps, {
      filePath: this.filePath,
      componentName: this.componentName
    });
    mobx.runInAction(() => {
      this.computePropsResponse = response;
    });
  }
  setInvocationSource(source) {
    this._invocationSource = source;
  }
  get defaultPropsSource() {
    return generateDefaultProps(this.computePropsResponse.types.props, this.computePropsResponse.types.all);
  }
  get propsType() {
    return preparePropsType(this.componentName, this.computePropsResponse.types.props, this.computePropsResponse.args, this.computePropsResponse.types.all);
  }
  get isStory() {
    return this.computePropsResponse.isStory;
  }
  get invocationSource() {
    const source = this._invocationSource;
    if (source === null) {
      return this.defaultInvocationSource;
    }
    return source;
  }
  get isDefaultInvocationSource() {
    return !this.invocationSource || this.invocationSource === this.defaultInvocationSource;
  }
  get defaultInvocationSource() {
    return generateInvocation(this.computePropsResponse.types.props, [
      ...extractFunctionKeys(this.computePropsResponse.types.props, this.computePropsResponse.types.all),
      ...this.computePropsResponse.args
    ], this.computePropsResponse.types.all);
  }
}

const REFRESH_PERIOD_MILLIS = 5e3;
class PreviewState {
  constructor(localApi, webApi, persistedStateController, options = {}) {
    this.localApi = localApi;
    this.persistedStateController = persistedStateController;
    this.options = options;
    this.actionLogs = new ActionLogsState();
    this.consoleLogs = new ConsolePanelState();
    this.reachable = true;
    this.component = null;
    this.appInfo = null;
    this.iframeRef = {
      current: null
    };
    this.cachedInvocations = {};
    this.pingInterval = null;
    this.messageListener = (event) => {
      const data = event.data;
      if (data && data.kind === "navigate") {
        window.__previewjs_navigate(data.componentId);
      }
    };
    this.popStateListener = () => {
      this.onUrlChanged().catch(console.error);
    };
    this.keydownListener = (e) => {
      if (!navigator.userAgent.includes(" Code/")) {
        return;
      }
      const hasMeta = e.ctrlKey || e.metaKey;
      if (!hasMeta) {
        return;
      }
      switch (e.key.toLowerCase()) {
        case "c":
          document.execCommand("copy");
          e.preventDefault();
          break;
        case "x":
          document.execCommand("cut");
          e.preventDefault();
          break;
        case "v":
          document.execCommand("paste");
          e.preventDefault();
          break;
        case "a":
          document.execCommand("selectAll");
          e.preventDefault();
          break;
      }
    };
    this.updateBanner = new UpdateBannerState(webApi, this.persistedStateController);
    this.iframeController = iframe.createController({
      getIframe: () => this.iframeRef.current,
      listener: (event) => {
        mobx.runInAction(() => {
          if (!this.component?.details) {
            return;
          }
          switch (event.kind) {
            case "bootstrapped":
            case "before-render":
              this.consoleLogs.onClear();
              break;
            case "update":
              this.consoleLogs.onClear();
              if (event.rendering?.kind === "success") {
                this.component.variantKey = event.rendering.variantKey;
                this.component.details.variants = event.rendering.variants;
              }
              break;
            case "log-message":
              this.consoleLogs.onConsoleMessage(event);
              break;
            case "action":
              this.actionLogs.onAction(event);
              break;
            default:
              throw assertNever__default(event);
          }
        });
      }
    });
    mobx.makeAutoObservable(this, {
      iframeRef: mobx.observable.ref,
      pingInterval: mobx.observable.ref,
      appInfo: mobx.observable.ref
    });
  }
  async start() {
    window.__previewjs_navigate = (componentId, variantKey) => {
      history.pushState(null, "", `/?p=${componentId}${variantKey ? `&v=${variantKey}` : ""}`);
      this.onUrlChanged().catch(console.error);
    };
    window.addEventListener("message", this.messageListener);
    window.addEventListener("popstate", this.popStateListener);
    this.iframeController.start();
    await this.onUrlChanged();
    this.pingInterval = setInterval(() => {
      this.ping().then(() => this.component?.details?.props.refresh()).catch(console.error);
    }, REFRESH_PERIOD_MILLIS);
    document.addEventListener("keydown", this.keydownListener);
    const { appInfo } = await this.localApi.request(api.localEndpoints.GetInfo);
    mobx.runInAction(() => {
      this.appInfo = appInfo;
    });
    await this.persistedStateController.start();
    await this.updateBanner.start(appInfo);
  }
  stop() {
    if (this.pingInterval) {
      clearInterval(this.pingInterval);
      this.pingInterval = null;
    }
    this.iframeController.stop();
    window.removeEventListener("message", this.messageListener);
    window.removeEventListener("popstate", this.popStateListener);
    document.removeEventListener("keydown", this.keydownListener);
  }
  setIframeRef(iframeRef) {
    if (iframeRef === this.iframeRef) {
      return;
    }
    this.iframeRef = iframeRef;
    this.renderComponent();
  }
  setComponent(componentId) {
    if (componentId === this.component?.componentId) {
      this.setVariant("custom");
    } else {
      window.__previewjs_navigate(componentId);
    }
  }
  setVariant(variantKey) {
    if (!this.component) {
      return;
    }
    window.__previewjs_navigate(this.component.componentId, variantKey);
  }
  updateProps(source) {
    if (!this.component?.details) {
      return;
    }
    this.component.details.props.setInvocationSource(source);
    this.cachedInvocations[this.component.componentId] = source;
    this.renderComponent();
  }
  resetProps() {
    if (!this.component?.details) {
      return;
    }
    this.component.details.props.setInvocationSource(null);
    delete this.cachedInvocations[this.component.componentId];
    this.renderComponent();
  }
  async onUrlChanged() {
    const urlParams = new URLSearchParams(document.location.search);
    const componentId = urlParams.get("p") || "";
    const variantKey = urlParams.get("v") || null;
    const decodedComponentId = decodeComponentId(componentId);
    if (!decodedComponentId.component) {
      mobx.runInAction(() => {
        this.component = null;
      });
      if (this.options.onFileChanged) {
        await this.options.onFileChanged(decodedComponentId.currentFilePath);
      }
      return;
    }
    const name = decodedComponentId.component.name;
    if (this.component?.componentId === componentId) {
      mobx.runInAction(() => {
        if (this.component) {
          this.component.variantKey = variantKey;
        }
      });
    } else {
      this.iframeController.showLoading();
      mobx.runInAction(() => {
        this.component = {
          componentId,
          name,
          variantKey,
          details: null
        };
      });
      if (this.options.onFileChanged) {
        await this.options.onFileChanged(decodedComponentId.currentFilePath);
      }
      const filePath = decodedComponentId.component.filePath;
      const props = new ComponentProps(this.localApi, filePath, name, this.cachedInvocations[componentId] || null);
      await props.refresh();
      mobx.runInAction(() => {
        this.component = {
          componentId,
          name,
          variantKey,
          details: {
            filePath,
            variants: null,
            props
          }
        };
      });
    }
    this.renderComponent();
  }
  renderComponent() {
    if (!this.component?.details) {
      return;
    }
    this.consoleLogs.onClear();
    this.iframeController.loadComponent({
      componentName: this.component.name,
      filePath: this.component.details.filePath,
      variantKey: this.component.variantKey,
      customVariantPropsSource: this.component.details.props.invocationSource,
      defaultPropsSource: this.component.details.props.defaultPropsSource
    });
  }
  async ping() {
    let reachable;
    try {
      await fetch("/ping/");
      reachable = true;
    } catch (e) {
      console.warn(e);
      reachable = false;
    }
    mobx.runInAction(() => {
      this.reachable = reachable;
    });
  }
}

exports.Link = Link;
exports.PersistedStateController = PersistedStateController;
exports.Preview = Preview;
exports.PreviewState = PreviewState;
exports.Selection = Selection;
exports.createLocalApi = createLocalApi;
exports.createWebApi = createWebApi;
