import { Api, PersistedState, ResponseOf, localEndpoints, webEndpoints } from '@previewjs/api';
import * as _previewjs_type_analyzer from '@previewjs/type-analyzer';
import { ValueType, CollectedTypes } from '@previewjs/type-analyzer';
import React$1 from 'react';
import { IconDefinition } from '@fortawesome/free-solid-svg-icons';
import { Action, LogMessage, PreviewIframeController, Variant } from '@previewjs/iframe';

declare function createLocalApi(url: string): Api;

declare function createWebApi(url: string): Api;

declare type PanelTab = {
    icon: IconDefinition;
    label: string;
    key: string;
    notificationCount: number;
    panel: React$1.ReactNode;
};

declare class ActionLogsState {
    logs: Item[];
    constructor();
    onAction(action: Action): void;
    private removeItem;
}
declare type Item = {
    key: string;
    action: Action;
    count: number;
    remove(): void;
};

interface LogMessageWithSuggestion extends LogMessage {
    suggestion?: Suggestion;
}
declare type Suggestion = {
    message: string;
    url: string;
};
declare class ConsolePanelState {
    logs: LogMessageWithSuggestion[];
    constructor();
    onConsoleMessage(event: LogMessage): void;
    onClear(): void;
    get unreadCount(): number;
}

declare class PersistedStateController {
    private readonly localApi;
    private persistedState;
    constructor(localApi: Api);
    start(): Promise<void>;
    get state(): PersistedState | null;
    update(stateUpdate: Partial<PersistedState>): Promise<void>;
}

declare global {
    interface Window {
        __previewjs_navigate(componentId: string, variantKey?: string | null): void;
    }
}

declare class UpdateBannerState {
    private readonly webApi;
    private readonly persistedStateController;
    private checkVersionResponse;
    constructor(webApi: Api, persistedStateController: PersistedStateController);
    start(appInfo: ResponseOf<typeof localEndpoints.GetInfo>["appInfo"]): Promise<void>;
    get update(): webEndpoints.UpdateAvailable | null;
    onUpdateDismissed: () => Promise<void>;
}

declare class ComponentProps {
    private readonly localApi;
    private readonly filePath;
    private readonly componentName;
    private computePropsResponse;
    private _invocationSource;
    constructor(localApi: Api, filePath: string, componentName: string, cachedInvocationSource: string | null);
    /**
     * Refreshes props type.
     *
     * IMPORTANT: This must be called successfully at least once before
     * any other method can be used.
     */
    refresh(): Promise<void>;
    setInvocationSource(source: string | null): void;
    /**
     * Source of default props that should be passed to the component.
     *
     * Typically this is "{}" (empty object) but when we know more about the component, we may
     * provide better defaults such as callback implementations, e.g. "{ onClick: () => {} }".
     *
     * Unlike defaultInvocation, defaultProps is not shown to the user.
     */
    get defaultPropsSource(): string;
    get propsType(): {
        propsTypeName: string;
        types: _previewjs_type_analyzer.CollectedTypes;
    };
    get isStory(): boolean;
    get invocationSource(): string;
    get isDefaultInvocationSource(): boolean;
    /**
     * Default source of invocation, used to fill the initial content of the props editor (unless
     * a preconfigured variant is used).
     *
     * This is typically `properties = {};` unless we're able to infer information about the
     * component's props.
     */
    private get defaultInvocationSource();
}

declare class PreviewState {
    private readonly localApi;
    private readonly persistedStateController;
    private readonly options;
    readonly iframeController: PreviewIframeController;
    readonly actionLogs: ActionLogsState;
    readonly consoleLogs: ConsolePanelState;
    readonly updateBanner: UpdateBannerState;
    reachable: boolean;
    component: {
        /**
         * ID of the component, comprising of a file path and local component ID.
         *
         * Follows the following format: <filePath>:<file-relative id>
         *
         * For example, a component "Foo" in src/App.tsx will have the ID "src/App.tsx:Foo"
         */
        componentId: string;
        /**
         * Human-friendly component name, such as "Foo".
         *
         * Typically corresponds to the second part of component ID, but not necessarily.
         */
        name: string;
        /**
         * Key of the component's variant currently displayed.
         *
         * Set to "custom" when a preconfigured variant isn't being used.
         *
         * Null when the component is still loading and information about its variants hasn't been
         * loaded yet (see details.variants below).
         */
        variantKey: string | null;
        details: {
            /**
             * File path where the component is loaded from. This typically corresponds to the first part
             * of the component ID, but not necessarily (e.g. storybook stories in a different file).
             */
            filePath: string;
            props: ComponentProps;
            /**
             * List of preconfigured variants for the component, if any.
             *
             * Null while the list of variants hasn't yet been loaded (this may only happen at runtime).
             */
            variants: Variant[] | null;
        } | null;
    } | null;
    appInfo: ResponseOf<typeof localEndpoints.GetInfo>["appInfo"] | null;
    private iframeRef;
    private cachedInvocations;
    private pingInterval;
    constructor(localApi: Api, webApi: Api, persistedStateController: PersistedStateController, options?: {
        onFileChanged?: (filePath: string | null) => Promise<void>;
    });
    start(): Promise<void>;
    stop(): void;
    private messageListener;
    private popStateListener;
    private keydownListener;
    setIframeRef(iframeRef: React.RefObject<HTMLIFrameElement | null>): void;
    setComponent(componentId: string): void;
    setVariant(variantKey: string): void;
    updateProps(source: string): void;
    resetProps(): void;
    private onUrlChanged;
    private renderComponent;
    private ping;
}

declare const Preview: (({ state, appLabel, headerAddon, subheader, footer, panelTabs, PropsPanel, viewport, }: {
    state: PreviewState;
    appLabel: string;
    headerAddon?: React$1.ReactNode;
    subheader?: React$1.ReactNode;
    footer?: React$1.ReactNode;
    panelTabs?: PanelTab[] | undefined;
    PropsPanel?: React$1.ComponentType<PropsPanelProps> | undefined;
    viewport: React$1.ReactNode;
}) => JSX.Element) & {
    displayName: string;
};
declare type PropsPanelProps = {
    componentName: string;
    propsType: ValueType;
    types: CollectedTypes;
    source: string;
    onChange: (source: string) => void;
    onReset?: () => void;
    codeEditor: React$1.ReactNode;
};

declare const Selection: React$1.MemoExoticComponent<React$1.ForwardRefExoticComponent<Pick<{
    state: PreviewState;
    icon?: IconDefinition | undefined;
} & React$1.RefAttributes<HTMLDivElement>, "key" | "icon" | "state"> & React$1.RefAttributes<HTMLDivElement>>>;

declare global {
    interface Window {
        openInExternalBrowser?(url: string): void;
    }
}
declare const Link: (props: React$1.ComponentProps<"a">) => JSX.Element;

export { Link, PersistedStateController, Preview, PreviewState, Selection, createLocalApi, createWebApi };
